/*
LINKEDHASHMAP PERFEZIONA IL CONTRATTO DELLA HASHMAP, PERMETTENDO CHE L'ITERAZIONE
SUI SUOI ELEMENTI AVVENGA NELL'ORDINE DI INSERIMENTO.
IL COSTRUTTORE RICHIEDE IL NUMERO DI ELEMENTI, IL FATTORE DI CARICO ED UN BOOLEANO
	IL NUMERO DI ELEMENTI E' QUELLO INIZIALE DELLA TABELLA HASH
	IL FATTORE DI CARICO E' UN VALORE CHE REGOLAMENTA LE COLLISIONI:
		UNA COLLISIONE SI HA QUANDO UN OGGETTO RESTITUISCE LO STESSO CODICE
		HASH DI UN ALTRO, E QUINDI VA SULLA STESSA CASELLA (LISTA LINEARE
		DEGLI ELEMENTI IN COLLISIONE).
		sE LE CASELLE SONO TANTE, LA PROBABILITA' DI AVERE LISTE LINEARI 
		DI OGGETTI SCENDE, DI CONTRO PERO' PEGGIORANO LE PRESTAZIONI DELL'
		ITERAZIONE PERCHE' LE CASELLE SONO DI PIU'
	QUANDO IL NUMERO DEGLI ELEMENTI DI UNA TABELLA HASH SUPERA IL PRODOTTO
	TRA FATTORE DI CARICO E NUMERO DI CASELLE, LE CASELLE SONO RADDOPPIATE,
	E LA TABELLA VIENE RIORGANIZZATA (REHASING DEGLI ELEMENTI E LORO RIALLOCAZIONE
	PER SCARICARE LE LISTE LINEARI)
*/


import java.util.*;

/**
   This program demonstrates the use of a map with key type
   String and value type Employee.
*/
public class MapTest
{  
   public static void main(String[] args)
   {  
      Map staff = new LinkedHashMap(101,0.75F,true);
      staff.put("144-25-5464", new Employee("Angela Hung"));
      staff.put("567-24-2546", new Employee("Harry Hacker"));
      staff.put("157-62-7935", new Employee("Gary Cooper"));
      staff.put("456-62-5527", new Employee("Francesca Cruz"));

      // print all entries

      System.out.println(staff);

      // remove an entry

      staff.remove("567-24-2546");

      // replace an entry

      staff.put("456-62-5527", new Employee("Francesca Miller"));

      // look up a value

      System.out.println(staff.get("157-62-7935"));

      // iterate through all entries

      Set entries = staff.entrySet();
      Iterator iter = entries.iterator();
      while (iter.hasNext())
      {  
         Map.Entry entry = (Map.Entry)iter.next();
         Object key = entry.getKey();
         Object value = entry.getValue();
         System.out.println("key=" + key + ", value=" + value);
      }
   }
}

/**
   A minimalist employee class for testing purposes.
*/
class Employee
{ 
   /**
      Constructs an employee with $0 salary.
      @param n the employee name
   */
   public Employee(String n)
   {  
      name = n;
      salary = 0;
   }

   public String toString()
   {  
      return "[name=" + name + ", salary=" + salary + "]";
   }

   /**
      Sets the employee salary to a new value.
      @param s the new salary.
   */
   public void setSalary(double s)
   {  
      salary = s;
   }

   private String name;
   private double salary;
}